@@ -27,7 +27,7 @@ class PMA_Error_Handler
      *
      * @var array of PMA_Error
      */
-    protected $_errors = array();
+    protected $errors = array();
 
     /**
      * Constructor - set PHP error handler
@@ -35,7 +35,15 @@ class PMA_Error_Handler
      */
     public function __construct()
     {
-        set_error_handler(array($this, 'handleError'));
+        /**
+         * Do not set ourselves as error handler in case of testsuite.
+         *
+         * This behavior is not tested there and breaks other tests as they
+         * rely on PHPUnit doing it's own error handling which we break here.
+         */
+        if (!defined('TESTSUITE')) {
+            set_error_handler(array($this, 'handleError'));
+        }
     }
 
     /**
@@ -51,22 +59,34 @@ class PMA_Error_Handler
                 $_SESSION['errors'] = array();
             }
 
-            if ($GLOBALS['cfg']['Error_Handler']['gather']) {
+            if (isset($GLOBALS['cfg']['Error_Handler'])
+                && $GLOBALS['cfg']['Error_Handler']['gather']
+            ) {
                 // remember all errors
-                $_SESSION['errors'] = array_merge($_SESSION['errors'], $this->_errors);
+                $_SESSION['errors'] = array_merge(
+                    $_SESSION['errors'],
+                    $this->errors
+                );
             } else {
                 // remember only not displayed errors
-                foreach ($this->_errors as $key => $error) {
+                foreach ($this->errors as $key => $error) {
                     /**
-                     * We don't want to store all errors here as it would explode user
-                     * session. In case  you want them all set
+                     * We don't want to store all errors here as it would
+                     * explode user session. In case  you want them all set
                      * $GLOBALS['cfg']['Error_Handler']['gather'] to true
                      */
                     if (count($_SESSION['errors']) >= 20) {
-                        $error = new PMA_Error(0, __('Too many error messages, some are not displayed.'), __FILE__, __LINE__);
+                        $error = new PMA_Error(
+                            0,
+                            __('Too many error messages, some are not displayed.'),
+                            __FILE__,
+                            __LINE__
+                        );
                         $_SESSION['errors'][$error->getHash()] = $error;
-                    }
-                    if (($error instanceof PMA_Error) && ! $error->isDisplayed()) {
+                        break;
+                    } else if (($error instanceof PMA_Error)
+                        && ! $error->isDisplayed()
+                    ) {
                         $_SESSION['errors'][$key] = $error;
                     }
                 }
@@ -81,8 +101,8 @@ class PMA_Error_Handler
      */
     protected function getErrors()
     {
-        $this->_checkSavedErrors();
-        return $this->_errors;
+        $this->checkSavedErrors();
+        return $this->errors;
     }
 
     /**
@@ -90,14 +110,16 @@ class PMA_Error_Handler
      *
      * This calls the addError() function, escaping the error string
      *
-     * @param integer $errno
-     * @param string  $errstr
-     * @param string  $errfile
-     * @param integer $errline
+     * @param integer $errno   error number
+     * @param string  $errstr  error string
+     * @param string  $errfile error file
+     * @param integer $errline error line
+     *
+     * @return void
      */
     public function handleError($errno, $errstr, $errfile, $errline)
     {
-        $this->addError($errstr, $errno, $errfile, $errline, $escape=true);
+        $this->addError($errstr, $errno, $errfile, $errline, true);
     }
 
     /**
@@ -111,46 +133,53 @@ class PMA_Error_Handler
      * Do not use the context parameter as we want to avoid storing the
      * complete $GLOBALS inside $_SESSION['errors']
      *
-     * @param integer $errno
-     * @param string  $errstr
-     * @param string  $errfile
-     * @param integer $errline
-     * @param boolean $escape
+     * @param string  $errstr  error string
+     * @param integer $errno   error number
+     * @param string  $errfile error file
+     * @param integer $errline error line
+     * @param boolean $escape  whether to escape the error string
+     *
+     * @return void
      */
-    public function addError($errstr, $errno, $errfile, $errline, $escape=true)
+    public function addError($errstr, $errno, $errfile, $errline, $escape = true)
     {
         if ($escape) {
             $errstr = htmlspecialchars($errstr);
         }
         // create error object
-        $error = new PMA_Error($errno, $errstr, $errfile, $errline);
+        $error = new PMA_Error(
+            $errno,
+            $errstr,
+            $errfile,
+            $errline
+        );
 
         // do not repeat errors
-        $this->_errors[$error->getHash()] = $error;
+        $this->errors[$error->getHash()] = $error;
 
         switch ($error->getNumber()) {
-            case E_USER_NOTICE:
-            case E_USER_WARNING:
-            case E_STRICT:
-            case E_DEPRECATED:
-            case E_NOTICE:
-            case E_WARNING:
-            case E_CORE_WARNING:
-            case E_COMPILE_WARNING:
-            case E_USER_ERROR:
-            case E_RECOVERABLE_ERROR:
-                // just collect the error
-                // display is called from outside
-                break;
-            case E_ERROR:
-            case E_PARSE:
-            case E_CORE_ERROR:
-            case E_COMPILE_ERROR:
-            default:
-                // FATAL error, dislay it and exit
-                $this->_dispFatalError($error);
-                exit;
-                break;
+        case E_USER_NOTICE:
+        case E_USER_WARNING:
+        case E_STRICT:
+        case E_DEPRECATED:
+        case E_NOTICE:
+        case E_WARNING:
+        case E_CORE_WARNING:
+        case E_COMPILE_WARNING:
+        case E_USER_ERROR:
+        case E_RECOVERABLE_ERROR:
+            // just collect the error
+            // display is called from outside
+            break;
+        case E_ERROR:
+        case E_PARSE:
+        case E_CORE_ERROR:
+        case E_COMPILE_ERROR:
+        default:
+            // FATAL error, dislay it and exit
+            $this->dispFatalError($error);
+            exit;
+            break;
         }
     }
 
@@ -158,11 +187,13 @@ class PMA_Error_Handler
     /**
      * log error to configured log facility
      *
-     * @todo    finish!
-     * @param PMA_Error $error
+     * @param PMA_Error $error the error
+     *
      * @return bool
+     *
+     * @todo finish!
      */
-    protected function _logError($error)
+    protected function logError($error)
     {
         return error_log($error->getMessage());
     }
@@ -170,65 +201,88 @@ class PMA_Error_Handler
     /**
      * trigger a custom error
      *
-     * @param string  $errorInfo
-     * @param integer $errorNumber
-     * @param string  $file
-     * @param integer $line
+     * @param string  $errorInfo   error message
+     * @param integer $errorNumber error number
+     * @param string  $file        file name
+     * @param integer $line        line number
+     *
+     * @return void
      */
-    public function triggerError($errorInfo, $errorNumber = null, $file = null, $line = null)
-    {
-        // we could also extract file and line from backtrace and call handleError() directly
+    public function triggerError($errorInfo, $errorNumber = null,
+        $file = null, $line = null
+    ) {
+        // we could also extract file and line from backtrace
+        // and call handleError() directly
         trigger_error($errorInfo, $errorNumber);
     }
 
     /**
      * display fatal error and exit
      *
-     * @param PMA_Error $error
+     * @param PMA_Error $error the error
+     *
+     * @return void
      */
-    protected function _dispFatalError($error)
+    protected function dispFatalError($error)
     {
         if (! headers_sent()) {
-            $this->_dispPageStart($error);
+            $this->dispPageStart($error);
         }
         $error->display();
-        $this->_dispPageEnd();
+        $this->dispPageEnd();
         exit;
     }
 
     /**
      * display the whole error page with all errors
      *
+     * @return void
      */
     public function dispErrorPage()
     {
         if (! headers_sent()) {
-            $this->_dispPageStart();
+            $this->dispPageStart();
         }
         $this->dispAllErrors();
-        $this->_dispPageEnd();
+        $this->dispPageEnd();
     }
 
     /**
-     * display user errors not displayed
+     * Displays user errors not displayed
      *
+     * @return void
      */
     public function dispUserErrors()
     {
+        echo $this->getDispUserErrors();
+    }
+
+    /**
+     * Renders user errors not displayed
+     *
+     * @return string
+     */
+    public function getDispUserErrors()
+    {
+        $retval = '';
         foreach ($this->getErrors() as $error) {
             if ($error->isUserError() && ! $error->isDisplayed()) {
-                $error->display();
+                $retval .= $error->getDisplay();
             }
         }
+        return $retval;
     }
 
     /**
      * display HTML header
      *
-     * @param PMA_error $error
+     * @param PMA_error $error the error
+     *
+     * @return void
      */
-    protected function _dispPageStart($error = null)
+    protected function dispPageStart($error = null)
     {
+        PMA_Response::getInstance()->disable();
         echo '<html><head><title>';
         if ($error) {
             echo $error->getTitle();
@@ -241,8 +295,9 @@ class PMA_Error_Handler
     /**
      * display HTML footer
      *
+     * @return void
      */
-    protected function _dispPageEnd()
+    protected function dispPageEnd()
     {
         echo '</body></html>';
     }
@@ -250,6 +305,7 @@ class PMA_Error_Handler
     /**
      * display all errors regardless already displayed or user errors
      *
+     * @return void
      */
     public function dispAllErrors()
     {
@@ -259,43 +315,60 @@ class PMA_Error_Handler
     }
 
     /**
-     * display errors not displayed
+     * renders errors not displayed
      *
+     * @return void
      */
-    public function dispErrors()
+    public function getDispErrors()
     {
+        $retval = '';
         if ($GLOBALS['cfg']['Error_Handler']['display']) {
             foreach ($this->getErrors() as $error) {
                 if ($error instanceof PMA_Error) {
                     if (! $error->isDisplayed()) {
-                        $error->display();
+                        $retval .= $error->getDisplay();
                     }
                 } else {
+                    ob_start();
                     var_dump($error);
+                    $retval .= ob_get_contents();
+                    ob_end_clean();
                 }
             }
         } else {
-            $this->dispUserErrors();
+            $retval .= $this->getDispUserErrors();
         }
+        return $retval;
+    }
+
+    /**
+     * displays errors not displayed
+     *
+     * @return void
+     */
+    public function dispErrors()
+    {
+        echo $this->getDispErrors();
     }
 
     /**
      * look in session for saved errors
      *
+     * @return void
      */
-    protected function _checkSavedErrors()
+    protected function checkSavedErrors()
     {
         if (isset($_SESSION['errors'])) {
 
             // restore saved errors
             foreach ($_SESSION['errors'] as $hash => $error) {
-                if ($error instanceof PMA_Error && ! isset($this->_errors[$hash])) {
-                    $this->_errors[$hash] = $error;
+                if ($error instanceof PMA_Error && ! isset($this->errors[$hash])) {
+                    $this->errors[$hash] = $error;
                 }
             }
-            //$this->_errors = array_merge($_SESSION['errors'], $this->_errors);
+            //$this->errors = array_merge($_SESSION['errors'], $this->errors);
 
-            // delet stored errors
+            // delete stored errors
             $_SESSION['errors'] = array();
             unset($_SESSION['errors']);
         }
@@ -304,7 +377,7 @@ class PMA_Error_Handler
     /**
      * return count of errors
      *
-     * @return  integer number of errors occoured
+     * @return integer number of errors occoured
      */
     public function countErrors()
     {
@@ -314,7 +387,7 @@ class PMA_Error_Handler
     /**
      * return count of user errors
      *
-     * @return  integer number of user errors occoured
+     * @return integer number of user errors occoured
      */
     public function countUserErrors()
     {
@@ -333,7 +406,7 @@ class PMA_Error_Handler
     /**
      * whether use errors occured or not
      *
-     * @return  boolean
+     * @return boolean
      */
     public function hasUserErrors()
     {
@@ -343,7 +416,7 @@ class PMA_Error_Handler
     /**
      * whether errors occured or not
      *
-     * @return  boolean
+     * @return boolean
      */
     public function hasErrors()
     {
