@@ -32,7 +32,7 @@
  */
 
 /**
- * block attempts to directly run this script 
+ * block attempts to directly run this script
  */
 if (getcwd() == dirname(__FILE__)) {
     die('Attack stopped');
@@ -101,6 +101,13 @@ require './libraries/core.lib.php';
  */
 require './libraries/sanitizing.lib.php';
 
+/**
+ * Warning about mbstring.
+ */
+if (! function_exists('mb_detect_encoding')) {
+    PMA_warnMissingExtension('mbstring', $fatal = true);
+}
+
 /**
  * the PMA_Theme class
  */
@@ -131,14 +138,19 @@ require './libraries/Tracker.class.php';
  */
 require './libraries/Table.class.php';
 
-if (!defined('PMA_MINIMUM_COMMON')) {
+/**
+ * the PMA_Types class
+ */
+require './libraries/Types.class.php';
+
+if (! defined('PMA_MINIMUM_COMMON')) {
     /**
      * common functions
      */
-    include_once './libraries/common.lib.php';
+    include_once './libraries/Util.class.php';
 
     /**
-     * Java script escaping.
+     * JavaScript escaping.
      */
     include_once './libraries/js_escape.lib.php';
 
@@ -146,29 +158,16 @@ if (!defined('PMA_MINIMUM_COMMON')) {
      * Include URL/hidden inputs generating.
      */
     include_once './libraries/url_generating.lib.php';
+
+    /**
+     * Used to generate the page
+     */
+    include_once 'libraries/Response.class.php';
 }
 
 /******************************************************************************/
 /* start procedural code                       label_start_procedural         */
 
-/**
- * protect against possible exploits - there is no need to have so much variables
- */
-if (count($_REQUEST) > 1000) {
-    die(__('possible exploit'));
-}
-
-/**
- * Check for numeric keys
- * (if register_globals is on, numeric key can be found in $GLOBALS)
- */
-foreach ($GLOBALS as $key => $dummy) {
-    if (is_numeric($key)) {
-        die(__('numeric key detected'));
-    }
-}
-unset($dummy);
-
 /**
  * PATH_INFO could be compromised if set, so remove it from PHP_SELF
  * and provide a clean PHP_SELF here
@@ -249,7 +248,8 @@ if (isset($_POST['usesubform'])) {
      * track this
      */
     if (isset($_POST['redirect'])
-      && $_POST['redirect'] != basename($PMA_PHP_SELF)) {
+        && $_POST['redirect'] != basename($PMA_PHP_SELF)
+    ) {
         $__redirect = $_POST['redirect'];
         unset($_POST['redirect']);
     }
@@ -278,24 +278,18 @@ if (version_compare(phpversion(), '5.4', 'lt')) {
     }
 }
 
-/**
- * include deprecated grab_globals only if required
- */
-if (empty($__redirect) && !defined('PMA_NO_VARIABLES_IMPORT')) {
-    include './libraries/grab_globals.lib.php';
-}
-
 /**
  * check timezone setting
  * this could produce an E_STRICT - but only once,
  * if not done here it will produce E_STRICT on every date/time function
+ * (starting with PHP 5.3, this code can produce E_WARNING rather than
+ *  E_STRICT)
  *
- * @todo need to decide how we should handle this (without @)
  */
 date_default_timezone_set(@date_default_timezone_get());
 
 /******************************************************************************/
-/* parsing configuration file                         LABEL_parsing_config_file      */
+/* parsing configuration file                  LABEL_parsing_config_file      */
 
 /**
  * We really need this one!
@@ -304,6 +298,13 @@ if (! function_exists('preg_replace')) {
     PMA_warnMissingExtension('pcre', true);
 }
 
+/**
+ * JSON is required in several places.
+ */
+if (! function_exists('json_encode')) {
+    PMA_warnMissingExtension('json', true);
+}
+
 /**
  * @global PMA_Config $GLOBALS['PMA_Config']
  * force reading of config file, because we removed sensitive values
@@ -327,8 +328,9 @@ $GLOBALS['PMA_Config']->enableBc();
  */
 $pma_cookie_version = 4;
 if (isset($_COOKIE)
- && (isset($_COOKIE['pmaCookieVer'])
-  && $_COOKIE['pmaCookieVer'] < $pma_cookie_version)) {
+    && (isset($_COOKIE['pmaCookieVer'])
+    && $_COOKIE['pmaCookieVer'] < $pma_cookie_version)
+) {
     // delete all cookies
     foreach ($_COOKIE as $cookie_name => $tmp) {
         $GLOBALS['PMA_Config']->removeCookie($cookie_name);
@@ -344,22 +346,8 @@ if (isset($_COOKIE)
 if ($GLOBALS['PMA_Config']->get('ForceSSL')
     && ! $GLOBALS['PMA_Config']->get('is_https')
 ) {
-    // grab current URL
-    $url = $GLOBALS['PMA_Config']->get('PmaAbsoluteUri');
-    // Parse current URL
-    $parsed = parse_url($url);
-    // In case parsing has failed do stupid string replacement
-    if ($parsed === false) {
-        // Replace http protocol
-        $url = preg_replace('@^http:@', 'https:', $url);
-    } else {
-        if($GLOBALS['PMA_Config']->get('SSLPort')) {
-            $port_number = $GLOBALS['PMA_Config']->get('SSLPort');
-        } else {
-            $port_number = 443;
-        }
-        $url = 'https://' . $parsed['host'] . ':' . $port_number . $parsed['path'];
-    }
+    // grab SSL URL
+    $url = $GLOBALS['PMA_Config']->getSSLUri();
     // Actually redirect
     PMA_sendHeaderLocation($url . PMA_generate_common_url($_GET, 'text'));
     // delete the current session, otherwise we get problems (see bug #2397877)
@@ -389,7 +377,6 @@ $GLOBALS['url_params'] = array();
  */
 $goto_whitelist = array(
     //'browse_foreigners.php',
-    //'calendar.php',
     //'changelog.php',
     //'chk_rel.php',
     'db_create.php',
@@ -405,18 +392,16 @@ $goto_whitelist = array(
     'db_printview.php',
     'db_search.php',
     'db_routines.php',
-    //'Documentation.html',
     'export.php',
     'import.php',
     //'index.php',
     //'navigation.php',
     //'license.php',
-    'main.php',
+    'index.php',
     'pdf_pages.php',
     'pdf_schema.php',
     //'phpinfo.php',
     'querywindow.php',
-    //'readme.php',
     'server_binlog.php',
     'server_collations.php',
     'server_databases.php',
@@ -426,10 +411,13 @@ $goto_whitelist = array(
     'server_privileges.php',
     'server_sql.php',
     'server_status.php',
+    'server_status_advisor.php',
+    'server_status_monitor.php',
+    'server_status_queries.php',
+    'server_status_variables.php',
     'server_variables.php',
     'sql.php',
     'tbl_addfield.php',
-    'tbl_alter.php',
     'tbl_change.php',
     'tbl_create.php',
     'tbl_import.php',
@@ -492,7 +480,12 @@ if (PMA_checkPageValidity($_REQUEST['back'], $goto_whitelist)) {
  * @todo variables should be handled by their respective owners (objects)
  * f.e. lang, server, collation_connection in PMA_Config
  */
-if (! PMA_isValid($_REQUEST['token']) || $_SESSION[' PMA_token '] != $_REQUEST['token']) {
+$token_mismatch = true;
+if (PMA_isValid($_REQUEST['token'])) {
+    $token_mismatch = ($_SESSION[' PMA_token '] != $_REQUEST['token']);
+}
+
+if ($token_mismatch) {
     /**
      *  List of parameters which are allowed from unsafe source
      */
@@ -507,11 +500,17 @@ if (! PMA_isValid($_REQUEST['token']) || $_SESSION[' PMA_token '] != $_REQUEST['
         'pma_lang', 'pma_collation_connection',
         /* Possible login form */
         'pma_servername', 'pma_username', 'pma_password',
-        /* for playing blobstreamable media */
-        'media_type', 'custom_type', 'bs_reference',
-        /* for changing BLOB repository file MIME type */
-        'bs_db', 'bs_table', 'bs_ref', 'bs_new_mime_type',
+        /* Needed to send the correct reply */
+        'ajax_request',
+        /* Permit to log out even if there is a token mismatch */
+        'old_usr'
     );
+    /**
+     * Allow changing themes in test/theme.php
+     */
+    if (defined('PMA_TEST_THEME')) {
+        $allow_list[] = 'set_theme';
+    }
     /**
      * Require cleanup functions
      */
@@ -569,49 +568,10 @@ if (PMA_isValid($_REQUEST['sql_query'])) {
     $GLOBALS['sql_query'] = $_REQUEST['sql_query'];
 }
 
-/**
- * avoid problems in phpmyadmin.css.php in some cases
- * @global string $js_frame
- */
-$_REQUEST['js_frame'] = PMA_ifSetOr($_REQUEST['js_frame'], '');
-
 //$_REQUEST['set_theme'] // checked later in this file LABEL_theme_setup
 //$_REQUEST['server']; // checked later in this file
 //$_REQUEST['lang'];   // checked by LABEL_loading_language_file
 
-
-/**
- * holds name of JavaScript files to be included in HTML header
- * @global array $js_include
- */
-$GLOBALS['js_include'] = array();
-$GLOBALS['js_include'][] = 'jquery/jquery-1.6.2+fix-9521.js';
-$GLOBALS['js_include'][] = 'jquery/jquery-ui-1.8.16.custom.js';
-$GLOBALS['js_include'][] = 'update-location.js';
-
-/**
- * holds an array of javascript code snippets to be included in the HTML header
- * Can be used with PMA_AddJSCode() to pass on js variables to the browser.
- * @global array $js_script
- */
-$GLOBALS['js_script'] = array();
-
-/**
- * Add common jQuery functions script here if necessary.
- */
-
-/**
- * JavaScript events that will be registered
- * @global array $js_events
- */
-$GLOBALS['js_events'] = array();
-
-/**
- * footnotes to be displayed ot the page bottom
- * @global array $footnotes
- */
-$GLOBALS['footnotes'] = array();
-
 /******************************************************************************/
 /* loading language file                       LABEL_loading_language_file    */
 
@@ -620,6 +580,15 @@ $GLOBALS['footnotes'] = array();
  */
 require './libraries/select_lang.lib.php';
 
+// Defines the cell alignment values depending on text direction
+if ($GLOBALS['text_dir'] == 'ltr') {
+    $GLOBALS['cell_align_left']  = 'left';
+    $GLOBALS['cell_align_right'] = 'right';
+} else {
+    $GLOBALS['cell_align_left']  = 'right';
+    $GLOBALS['cell_align_right'] = 'left';
+}
+
 /**
  * check for errors occurred while loading configuration
  * this check is done here after loading language files to present errors in locale
@@ -627,20 +596,25 @@ require './libraries/select_lang.lib.php';
 $GLOBALS['PMA_Config']->checkPermissions();
 
 if ($GLOBALS['PMA_Config']->error_config_file) {
-    $error = '<h1>' . __('Failed to read configuration file') . '</h1>'
-        . _('This usually means there is a syntax error in it, please check any errors shown below.')
-        . '<br />'
-        . '<br />'
-        . '<iframe src="show_config_errors.php" />';
+    $error = '[strong]' . __('Failed to read configuration file') . '[/strong]'
+        . '[br][br]'
+        . __('This usually means there is a syntax error in it, please check any errors shown below.')
+        . '[br][br]'
+        . '[conferr]';
     trigger_error($error, E_USER_ERROR);
 }
 if ($GLOBALS['PMA_Config']->error_config_default_file) {
-    $error = sprintf(__('Could not load default configuration from: %1$s'),
-        $GLOBALS['PMA_Config']->default_source);
+    $error = sprintf(
+        __('Could not load default configuration from: %1$s'),
+        $GLOBALS['PMA_Config']->default_source
+    );
     trigger_error($error, E_USER_ERROR);
 }
 if ($GLOBALS['PMA_Config']->error_pma_uri) {
-    trigger_error(__('The <tt>$cfg[\'PmaAbsoluteUri\']</tt> directive MUST be set in your configuration file!'), E_USER_ERROR);
+    trigger_error(
+        __('The [code]$cfg[\'PmaAbsoluteUri\'][/code] directive MUST be set in your configuration file!'),
+        E_USER_ERROR
+    );
 }
 
 
@@ -670,14 +644,23 @@ if (! isset($cfg['Servers']) || count($cfg['Servers']) == 0) {
 
         // Detect wrong configuration
         if (!is_int($server_index) || $server_index < 1) {
-            trigger_error(sprintf(__('Invalid server index: %s'), $server_index), E_USER_ERROR);
+            trigger_error(
+                sprintf(__('Invalid server index: %s'), $server_index),
+                E_USER_ERROR
+            );
         }
 
         $each_server = array_merge($default_server, $each_server);
 
         // Don't use servers with no hostname
         if ($each_server['connect_type'] == 'tcp' && empty($each_server['host'])) {
-            trigger_error(sprintf(__('Invalid hostname for server %1$s. Please review your configuration.'), $server_index), E_USER_ERROR);
+            trigger_error(
+                sprintf(
+                    __('Invalid hostname for server %1$s. Please review your configuration.'),
+                    $server_index
+                ),
+                E_USER_ERROR
+            );
         }
 
         // Final solution to bug #582890
@@ -685,7 +668,10 @@ if (! isset($cfg['Servers']) || count($cfg['Servers']) == 0) {
         // and there is nothing in the verbose server name
         // or the host field, then generate a name for the server
         // in the form of "Server 2", localized of course!
-        if ($each_server['connect_type'] == 'socket' && empty($each_server['host']) && empty($each_server['verbose'])) {
+        if ($each_server['connect_type'] == 'socket'
+            && empty($each_server['host'])
+            && empty($each_server['verbose'])
+        ) {
             $each_server['verbose'] = __('Server') . $server_index;
         }
 
@@ -760,13 +746,6 @@ $GLOBALS['pmaThemeImage']   = $_SESSION['PMA_Theme']->getImgPath();
  */
 if (@file_exists($_SESSION['PMA_Theme']->getLayoutFile())) {
     include $_SESSION['PMA_Theme']->getLayoutFile();
-    /**
-     * @todo remove if all themes are update use Navi instead of Left as frame name
-     */
-    if (! isset($GLOBALS['cfg']['NaviWidth'])
-     && isset($GLOBALS['cfg']['LeftWidth'])) {
-        $GLOBALS['cfg']['NaviWidth'] = $GLOBALS['cfg']['LeftWidth'];
-    }
 }
 
 if (! defined('PMA_MINIMUM_COMMON')) {
@@ -784,8 +763,10 @@ if (! defined('PMA_MINIMUM_COMMON')) {
      * Lookup server by name
      * (see FAQ 4.8)
      */
-    if (! empty($_REQUEST['server']) && is_string($_REQUEST['server'])
-     && ! is_numeric($_REQUEST['server'])) {
+    if (! empty($_REQUEST['server'])
+        && is_string($_REQUEST['server'])
+        && ! is_numeric($_REQUEST['server'])
+    ) {
         foreach ($cfg['Servers'] as $i => $server) {
             if ($server['host'] == $_REQUEST['server']) {
                 $_REQUEST['server'] = $i;
@@ -807,7 +788,11 @@ if (! defined('PMA_MINIMUM_COMMON')) {
      * and '$cfg['ServerDefault'] = 0' is set.
      */
 
-    if (isset($_REQUEST['server']) && (is_string($_REQUEST['server']) || is_numeric($_REQUEST['server'])) && ! empty($_REQUEST['server']) && ! empty($cfg['Servers'][$_REQUEST['server']])) {
+    if (isset($_REQUEST['server'])
+        && (is_string($_REQUEST['server']) || is_numeric($_REQUEST['server']))
+        && ! empty($_REQUEST['server'])
+        && ! empty($cfg['Servers'][$_REQUEST['server']])
+    ) {
         $GLOBALS['server'] = $_REQUEST['server'];
         $cfg['Server'] = $cfg['Servers'][$GLOBALS['server']];
     } else {
@@ -825,7 +810,8 @@ if (! defined('PMA_MINIMUM_COMMON')) {
      * Kanji encoding convert feature appended by Y.Kawada (2002/2/20)
      */
     if (function_exists('mb_convert_encoding')
-     && $lang == 'ja') {
+        && $lang == 'ja'
+    ) {
         include_once './libraries/kanji-encoding.lib.php';
     } // end if
 
@@ -837,7 +823,8 @@ if (! defined('PMA_MINIMUM_COMMON')) {
     if (isset($GLOBALS['collation_connection'])) {
         $GLOBALS['PMA_Config']->setCookie(
             'pma_collation_connection',
-            $GLOBALS['collation_connection']);
+            $GLOBALS['collation_connection']
+        );
     }
 
     $_SESSION['PMA_Theme_Manager']->setThemeCookie();
@@ -852,8 +839,8 @@ if (! defined('PMA_MINIMUM_COMMON')) {
         include_once './libraries/logging.lib.php';
 
         // get LoginCookieValidity from preferences cache
-        // no generic solution for loading preferences from cache as some settings need to be kept
-        // for processing in PMA_Config::loadUserPreferences()
+        // no generic solution for loading preferences from cache as some settings
+        // need to be kept for processing in PMA_Config::loadUserPreferences()
         $cache_key = 'server_' . $GLOBALS['server'];
         if (isset($_SESSION['cache'][$cache_key]['userprefs']['LoginCookieValidity'])) {
             $value = $_SESSION['cache'][$cache_key]['userprefs']['LoginCookieValidity'];
@@ -868,19 +855,31 @@ if (! defined('PMA_MINIMUM_COMMON')) {
 
         // to allow HTTP or http
         $cfg['Server']['auth_type'] = strtolower($cfg['Server']['auth_type']);
-        if (! file_exists('./libraries/auth/' . $cfg['Server']['auth_type'] . '.auth.lib.php')) {
-            PMA_fatalError(__('Invalid authentication method set in configuration:') . ' ' . $cfg['Server']['auth_type']);
-        }
+
         /**
          * the required auth type plugin
          */
-        include_once './libraries/auth/' . $cfg['Server']['auth_type'] . '.auth.lib.php';
-        if (!PMA_auth_check()) {
+        $auth_class = "Authentication" . ucfirst($cfg['Server']['auth_type']);
+        if (! file_exists('./libraries/plugins/auth/' . $auth_class . '.class.php')) {
+            PMA_fatalError(
+                __('Invalid authentication method set in configuration:')
+                . ' ' . $cfg['Server']['auth_type']
+            );
+        }
+        if (isset($_REQUEST['pma_password'])) {
+            $_REQUEST['pma_password'] = substr($_REQUEST['pma_password'], 0, 256);
+        }
+        include_once  './libraries/plugins/auth/' . $auth_class . '.class.php';
+        // todo: add plugin manager
+        $plugin_manager = null;
+        $auth_plugin = new $auth_class($plugin_manager);
+
+        if (! $auth_plugin->authCheck()) {
             /* Force generating of new session on login */
             PMA_secureSession();
-            PMA_auth();
+            $auth_plugin->auth();
         } else {
-            PMA_auth_set_user();
+            $auth_plugin->authSetUser();
         }
 
          // Check IP-based Allow/Deny rules as soon as possible to reject the
@@ -889,7 +888,8 @@ if (! defined('PMA_MINIMUM_COMMON')) {
         // http://cvs.apache.org/viewcvs.cgi/httpd-2.0/modules/aaa/mod_access.c?rev=1.37&content-type=text/vnd.viewcvs-markup
         // Look at: "static int check_dir_access(request_rec *r)"
         if (isset($cfg['Server']['AllowDeny'])
-                && isset($cfg['Server']['AllowDeny']['order'])) {
+            && isset($cfg['Server']['AllowDeny']['order'])
+        ) {
 
             /**
              * ip based access library
@@ -913,8 +913,7 @@ if (! defined('PMA_MINIMUM_COMMON')) {
                     $allowDeny_forbidden = false;
                 }
             } elseif ($cfg['Server']['AllowDeny']['order'] == 'explicit') {
-                if (PMA_allowDeny('allow')
-                  && !PMA_allowDeny('deny')) {
+                if (PMA_allowDeny('allow') && ! PMA_allowDeny('deny')) {
                     $allowDeny_forbidden = false;
                 } else {
                     $allowDeny_forbidden = true;
@@ -924,25 +923,22 @@ if (! defined('PMA_MINIMUM_COMMON')) {
             // Ejects the user if banished
             if ($allowDeny_forbidden) {
                 PMA_log_user($cfg['Server']['user'], 'allow-denied');
-                PMA_auth_fails();
+                $auth_plugin->authFails();
             }
-            unset($allowDeny_forbidden); //Clean up after you!
         } // end if
 
         // is root allowed?
         if (!$cfg['Server']['AllowRoot'] && $cfg['Server']['user'] == 'root') {
             $allowDeny_forbidden = true;
             PMA_log_user($cfg['Server']['user'], 'root-denied');
-            PMA_auth_fails();
-            unset($allowDeny_forbidden); //Clean up after you!
+            $auth_plugin->authFails();
         }
 
         // is a login without password allowed?
         if (!$cfg['Server']['AllowNoPassword'] && $cfg['Server']['password'] == '') {
             $login_without_password_is_forbidden = true;
             PMA_log_user($cfg['Server']['user'], 'empty-denied');
-            PMA_auth_fails();
-            unset($login_without_password_is_forbidden); //Clean up after you!
+            $auth_plugin->authFails();
         }
 
         // if using TCP socket is not needed
@@ -957,24 +953,32 @@ if (! defined('PMA_MINIMUM_COMMON')) {
         $controllink = false;
         if ($cfg['Server']['controluser'] != '') {
             if (! empty($cfg['Server']['controlhost'])) {
-                $controllink = PMA_DBI_connect($cfg['Server']['controluser'],
-                    $cfg['Server']['controlpass'], true,
+                $controllink = PMA_DBI_connect(
+                    $cfg['Server']['controluser'],
+                    $cfg['Server']['controlpass'],
+                    true,
                     array('host' => $cfg['Server']['controlhost'])
                 );
             } else {
-                $controllink = PMA_DBI_connect($cfg['Server']['controluser'],
-                    $cfg['Server']['controlpass'], true);
+                $controllink = PMA_DBI_connect(
+                    $cfg['Server']['controluser'],
+                    $cfg['Server']['controlpass'],
+                    true
+                );
             }
         }
 
         // Connects to the server (validates user's login)
-        $userlink = PMA_DBI_connect($cfg['Server']['user'],
-            $cfg['Server']['password'], false);
+        $userlink = PMA_DBI_connect(
+            $cfg['Server']['user'], $cfg['Server']['password'], false
+        );
 
         if (! $controllink) {
             $controllink = $userlink;
         }
 
+        $auth_plugin->storeUserCredentials();
+
         /* Log success */
         PMA_log_user($cfg['Server']['user']);
 
@@ -984,7 +988,19 @@ if (! defined('PMA_MINIMUM_COMMON')) {
          *  - > 5.0.15
          */
         if (PMA_MYSQL_INT_VERSION < 50015) {
-            PMA_fatalError(__('You should upgrade to %s %s or later.'), array('MySQL', '5.0.15'));
+            PMA_fatalError(
+                __('You should upgrade to %s %s or later.'),
+                array('MySQL', '5.0.15')
+            );
+        }
+
+        /**
+         * Type handling object.
+         */
+        if (PMA_DRIZZLE) {
+            $GLOBALS['PMA_Types'] = new PMA_Types_Drizzle();
+        } else {
+            $GLOBALS['PMA_Types'] = new PMA_Types_MySQL();
         }
 
         if (PMA_DRIZZLE) {
@@ -1016,7 +1032,9 @@ if (! defined('PMA_MINIMUM_COMMON')) {
         /**
          * some resetting has to be done when switching servers
          */
-        if (isset($_SESSION['tmp_user_values']['previous_server']) && $_SESSION['tmp_user_values']['previous_server'] != $GLOBALS['server']) {
+        if (isset($_SESSION['tmp_user_values']['previous_server'])
+            && $_SESSION['tmp_user_values']['previous_server'] != $GLOBALS['server']
+        ) {
             unset($_SESSION['tmp_user_values']['navi_limit_offset']);
         }
         $_SESSION['tmp_user_values']['previous_server'] = $GLOBALS['server'];
@@ -1027,20 +1045,45 @@ if (! defined('PMA_MINIMUM_COMMON')) {
      * check if profiling was requested and remember it
      * (note: when $cfg['ServerDefault'] = 0, constant is not defined)
      */
-    if (isset($_REQUEST['profiling']) && PMA_profilingSupported()) {
+    if (isset($_REQUEST['profiling'])
+        && PMA_Util::profilingSupported()
+    ) {
         $_SESSION['profiling'] = true;
     } elseif (isset($_REQUEST['profiling_form'])) {
         // the checkbox was unchecked
         unset($_SESSION['profiling']);
     }
+    /**
+     * Inclusion of profiling scripts is needed on various
+     * pages like sql, tbl_sql, db_sql, tbl_select
+     */
+    if (! defined('PMA_BYPASS_GET_INSTANCE')) {
+        $response = PMA_Response::getInstance();
+    }
+    if (isset($_SESSION['profiling'])) {
+        $header   = $response->getHeader();
+        $scripts  = $header->getScripts();
+        /* < IE 9 doesn't support canvas natively */
+        if (PMA_USR_BROWSER_AGENT == 'IE' && PMA_USR_BROWSER_VER < 9) {
+            $scripts->addFile('canvg/flashcanvas.js');
+        }
+        $scripts->addFile('jqplot/jquery.jqplot.js');
+        $scripts->addFile('jqplot/plugins/jqplot.pieRenderer.js');
+        $scripts->addFile('canvg/canvg.js');
+    }
 
-    // library file for blobstreaming
-    include_once './libraries/blobstreaming.lib.php';
-
-    // checks for blobstreaming plugins and databases that support
-    // blobstreaming (by having the necessary tables for blobstreaming)
-    checkBLOBStreamingPlugins();
-
+    /*
+     * There is no point in even attempting to process
+     * an ajax request if there is a token mismatch
+     */
+    if (isset($response) && $response->isAjax() && $token_mismatch) {
+        $response->isSuccess(false);
+        $response->addJSON(
+            'message',
+            PMA_Message::error(__('Error: Token mismatch'))
+        );
+        exit;
+    }
 } // end if !defined('PMA_MINIMUM_COMMON')
 
 // load user preferences
@@ -1080,6 +1123,32 @@ if (isset($_REQUEST['grid_edit']) && $_REQUEST['grid_edit'] == true) {
     $GLOBALS['grid_edit'] = false;
 }
 
+if (isset($_REQUEST['GLOBALS']) || isset($_FILES['GLOBALS'])) {
+    PMA_fatalError(__("GLOBALS overwrite attempt"));
+}
+
+/**
+ * protect against possible exploits - there is no need to have so much variables
+ */
+if (count($_REQUEST) > 1000) {
+    PMA_fatalError(__('possible exploit'));
+}
+
+/**
+ * Check for numeric keys
+ * (if register_globals is on, numeric key can be found in $GLOBALS)
+ */
+foreach ($GLOBALS as $key => $dummy) {
+    if (is_numeric($key)) {
+        PMA_fatalError(__('numeric key detected'));
+    }
+}
+unset($dummy);
+
+// here, the function does not exist with this configuration:
+// $cfg['ServerDefault'] = 0;
+$GLOBALS['is_superuser'] = function_exists('PMA_isSuperuser') && PMA_isSuperuser();
+
 if (!empty($__redirect) && in_array($__redirect, $goto_whitelist)) {
     /**
      * include subform target page
@@ -1087,4 +1156,5 @@ if (!empty($__redirect) && in_array($__redirect, $goto_whitelist)) {
     include $__redirect;
     exit();
 }
+
 ?>
